package com.chelvc.framework.jpush.context;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Collectors;

import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.Options;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.AndroidNotification;
import cn.jpush.api.push.model.notification.IosAlert;
import cn.jpush.api.push.model.notification.IosNotification;
import cn.jpush.api.push.model.notification.Notification;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.jpush.config.JPushProperties;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import lombok.NonNull;

/**
 * JPush上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class JPushContextHolder {
    /**
     * VIVO标识
     */
    public static final String VIVO = "vivo";

    /**
     * OPPO标识
     */
    public static final String OPPO = "oppo";

    /**
     * 荣耀标识
     */
    public static final String HONOR = "honor";

    /**
     * 小米标识
     */
    public static final String XIAOMI = "xiaomi";

    /**
     * 华为标识
     */
    public static final String HUAWEI = "huawei";

    /**
     * 安卓通知消息工厂
     */
    private static class AndroidNotificationBuilder extends AndroidNotification.Builder {
        private JsonObject intent;

        @Override
        public AndroidNotification.Builder setIntent(JsonObject intent) {
            return super.setIntent(this.intent = intent);
        }
    }

    /**
     * 安卓通知参数选项
     */
    private static volatile Options ANDROID_OPTIONS;

    private JPushContextHolder() {
    }

    /**
     * 将对象转换成别名
     *
     * @param objects 对象数组
     * @return 别名数组
     */
    public static Audience alias(Object... objects) {
        if (objects == null || objects.length == 0) {
            return null;
        }
        List<String> aliases = Lists.newArrayListWithCapacity(objects.length);
        for (Object object : objects) {
            if (object != null) {
                aliases.add(String.valueOf(object));
            }
        }
        return aliases.isEmpty() ? null : Audience.alias(aliases);
    }

    /**
     * 将对象转换成别名
     *
     * @param objects 对象集合
     * @return 别名数组
     */
    public static Audience alias(Collection<?> objects) {
        if (ObjectUtils.isEmpty(objects)) {
            return null;
        }
        List<String> aliases = objects.stream().filter(Objects::nonNull).map(String::valueOf)
                .collect(Collectors.toList());
        return aliases.isEmpty() ? null : Audience.alias(aliases);
    }

    /**
     * 平台信息适配
     *
     * @param platform 平台信息
     * @return 平台信息
     */
    public static cn.jpush.api.push.model.Platform adapter(Platform platform) {
        if (platform == Platform.IOS) {
            return cn.jpush.api.push.model.Platform.ios();
        } else if (platform == Platform.ANDROID) {
            return cn.jpush.api.push.model.Platform.android();
        }
        return null;
    }

    /**
     * 构建通知对象
     *
     * @param platform 平台信息
     * @param title    通知标题
     * @param content  通知内容
     * @return 通知对象实例
     */
    public static Notification notification(@NonNull com.chelvc.framework.common.model.Platform platform,
                                            @NonNull String title, @NonNull String content) {
        return notification(platform, title, content, Collections.emptyMap());
    }

    /**
     * 构建通知对象
     *
     * @param platform 平台信息
     * @param title    通知标题
     * @param content  通知内容
     * @param extras   扩展信息
     * @return 通知对象实例
     */
    public static Notification notification(@NonNull com.chelvc.framework.common.model.Platform platform,
                                            @NonNull String title, @NonNull String content,
                                            @NonNull Map<String, String> extras) {
        return notification(platform, title, content, extras, builder -> builder, builder -> builder);
    }

    /**
     * 构建通知对象
     *
     * @param platform 平台信息
     * @param title    通知标题
     * @param content  通知内容
     * @param ios      苹果通知构造回调函数
     * @param android  安卓通知构造回调函数
     * @return 通知对象实例
     */
    public static Notification notification(
            @NonNull com.chelvc.framework.common.model.Platform platform,
            @NonNull String title, @NonNull String content,
            @NonNull Function<IosNotification.Builder, IosNotification.Builder> ios,
            @NonNull Function<AndroidNotification.Builder, AndroidNotification.Builder> android) {
        return notification(platform, title, content, Collections.emptyMap(), ios, android);
    }

    /**
     * 构建通知对象
     *
     * @param platform 平台信息
     * @param title    通知标题
     * @param content  通知内容
     * @param extras   扩展信息
     * @param ios      苹果通知构造回调函数
     * @param android  安卓通知构造回调函数
     * @return 通知对象实例
     */
    public static Notification notification(
            @NonNull com.chelvc.framework.common.model.Platform platform,
            @NonNull String title, @NonNull String content, @NonNull Map<String, String> extras,
            @NonNull Function<IosNotification.Builder, IosNotification.Builder> ios,
            @NonNull Function<AndroidNotification.Builder, AndroidNotification.Builder> android) {
        if (platform == com.chelvc.framework.common.model.Platform.IOS) {
            IosNotification.Builder builder = IosNotification.newBuilder()
                    //设置ios标题
                    .setAlert(IosAlert.newBuilder().setTitleAndBody(title, null, content).build())
                    // 设置扩展参数
                    .addExtras(extras)
                    //如果要设置右侧大图，推送的时候携带 "mutable-content":true 说明是支持 iOS10 的 UNNotificationServiceExtension
                    .setMutableContent(true)
                    //如果不需要自定义声音，推送的时候，iOS 的 sound 值保持默认，传 default ，不要传空！
                    .setSound("default")
                    //设置角标自动+1
                    .incrBadge(1);
            return Notification.newBuilder().addPlatformNotification(ios.apply(builder).build()).build();
        } else if (platform == com.chelvc.framework.common.model.Platform.ANDROID) {
            AndroidNotification.Builder builder = new AndroidNotificationBuilder()
                    // 设置标题
                    .setTitle(title)
                    // 设置通知内容
                    .setAlert(content)
                    // 设置扩展参数
                    .addExtras(extras);
            builder = android.apply(builder);

            // 设置默认 Intent
            JsonObject intent = ObjectUtils.ifNull(((AndroidNotificationBuilder) builder).intent, JsonObject::new);
            if (!intent.has("url")) {
                intent.add("url", new JsonPrimitive("intent:#Intent;action=android.intent.action.MAIN;end"));
                builder.setIntent(intent);
            }
            return Notification.newBuilder().addPlatformNotification(builder.build()).build();
        }
        throw new UnsupportedOperationException("Not support platform: " + platform.name());
    }

    /**
     * 获取安卓通知可选参数
     *
     * @return 安卓通知参数选项
     */
    public static Options getAndroidNotifyOptions() {
        if (ANDROID_OPTIONS == null) {
            synchronized (Options.class) {
                if (ANDROID_OPTIONS == null) {
                    ANDROID_OPTIONS = getAndroidNotifyOptions((brand, channel) -> {
                    });
                }
            }
        }
        return ANDROID_OPTIONS;
    }

    /**
     * 获取安卓通知可选参数
     *
     * @param consumer 手机品牌/配置回调函数
     * @return 推送可选项
     */
    public static Options getAndroidNotifyOptions(@NonNull BiConsumer<String, JsonObject> consumer) {
        JPushProperties properties = ApplicationContextHolder.getBean(JPushProperties.class);
        JsonObject vivo = initializeAndroidChannel(properties.getVivo());
        consumer.accept(VIVO, vivo);
        JsonObject oppo = initializeAndroidChannel(properties.getOppo());
        consumer.accept(OPPO, oppo);
        JsonObject honor = initializeAndroidChannel(properties.getHonor());
        consumer.accept(HONOR, honor);
        JsonObject xiaomi = initializeAndroidChannel(properties.getXiaomi());
        consumer.accept(XIAOMI, xiaomi);
        JsonObject huawei = initializeAndroidChannel(properties.getHuawei());
        consumer.accept(HUAWEI, huawei);
        return Options.newBuilder().setThirdPartyChannelV2(ImmutableMap.of(
                VIVO, vivo, OPPO, oppo, HONOR, honor, XIAOMI, xiaomi, HUAWEI, huawei
        )).build();
    }

    /**
     * 初始化安卓渠道
     *
     * @param attribute 配置属性
     * @return 渠道信息
     */
    public static JsonObject initializeAndroidChannel(@NonNull JPushProperties.Attribute attribute) {
        JsonObject channel = new JsonObject();
        if (StringUtils.notEmpty(attribute.getScene())) {
            channel.addProperty("category", attribute.getScene());
        }
        if (StringUtils.notEmpty(attribute.getChannel())) {
            channel.addProperty("channel_id", attribute.getChannel());
        }
        if (StringUtils.notEmpty(attribute.getImportance())) {
            channel.addProperty("importance", attribute.getImportance());
        }
        if (StringUtils.notEmpty(attribute.getDistribution())) {
            channel.addProperty("distribution", attribute.getDistribution());
        }
        if (Objects.nonNull(attribute.getClassification())) {
            channel.addProperty("classification", attribute.getClassification());
        }
        return channel;
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param audience 目标群体
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Message message,
                                      @NonNull Audience audience) {
        return new PushPayload.Builder().setPlatform(adapter(platform)).setMessage(message)
                .setAudience(audience).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名数组
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Message message,
                                      @NonNull String... aliases) {
        return payload(platform, message, Audience.alias(aliases));
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名集合
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Message message,
                                      @NonNull Collection<String> aliases) {
        return payload(platform, message, Audience.alias(aliases));
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param audience     目标群体
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull Audience audience) {
        Options options = platform == Platform.ANDROID ? getAndroidNotifyOptions() : null;
        return payload(platform, notification, audience, options);
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param audience     目标群体
     * @param options      通知可选项
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull Audience audience, Options options) {
        return new PushPayload.Builder().setPlatform(adapter(platform)).setNotification(notification)
                .setAudience(audience).setOptions(options).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名数组
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull String... aliases) {
        return payload(platform, notification, Audience.alias(aliases));
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param options      通知可选项
     * @param aliases      用户别名数组
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      Options options, @NonNull String... aliases) {
        return payload(platform, notification, Audience.alias(aliases), options);
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名集合
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull Collection<String> aliases) {
        return payload(platform, notification, Audience.alias(aliases));
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param options      通知可选项
     * @param aliases      用户别名集合
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      Options options, @NonNull Collection<String> aliases) {
        return payload(platform, notification, Audience.alias(aliases), options);
    }

    /**
     * 构建消息推送载体对象
     *
     * @param target   目标用户
     * @param platform 目标平台
     * @param message  透传消息
     * @return 消息推送载体
     */
    public static PushPayload target(@NonNull String target, @NonNull Platform platform, @NonNull Message message) {
        return new PushPayload.Builder().setTarget(target).setPlatform(adapter(platform)).setMessage(message).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param target       目标用户
     * @param platform     目标平台
     * @param notification 推送通知
     * @return 消息推送载体
     */
    public static PushPayload target(@NonNull String target, @NonNull Platform platform,
                                     @NonNull Notification notification) {
        Options options = platform == Platform.ANDROID ? getAndroidNotifyOptions() : null;
        return target(target, platform, notification, options);
    }

    /**
     * 构建消息推送载体对象
     *
     * @param target       目标用户
     * @param platform     目标平台
     * @param notification 推送通知
     * @param options      通知可选项
     * @return 消息推送载体
     */
    public static PushPayload target(@NonNull String target, @NonNull Platform platform,
                                     @NonNull Notification notification, Options options) {
        return new PushPayload.Builder().setTarget(target).setPlatform(adapter(platform))
                .setNotification(notification).setOptions(options).build();
    }
}
