package com.chelvc.framework.rocketmq.consumer;

import java.lang.reflect.Type;
import java.time.Duration;
import java.util.Collections;
import java.util.List;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.consumer.FilterExpression;
import org.apache.rocketmq.client.apis.consumer.FilterExpressionType;
import org.apache.rocketmq.client.apis.consumer.SimpleConsumer;
import org.apache.rocketmq.client.apis.consumer.SimpleConsumerBuilder;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.springframework.core.env.Environment;

/**
 * 消息批量消费监听器容器
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class MultipleRocketMQListenerContainer<T> implements RocketMQListenerContainer<T> {
    private int batch;
    private Type type;
    private String group;
    private Thread thread;
    private Duration duration;
    private SimpleConsumer consumer;
    private RocketMQListener<T> listener;
    private volatile boolean running = true;

    /**
     * 接收消息
     *
     * @return 消息视图列表
     */
    private List<MessageView> receive() {
        try {
            return this.consumer.receive(this.batch, this.duration);
        } catch (Throwable t) {
            if (t.getCause() instanceof InterruptedException) {
                Thread.currentThread().interrupt();
            } else {
                log.error("RocketMQ message receive failed: {}", this.group, t);
            }
        }
        return Collections.emptyList();
    }

    /**
     * 批量消费消息
     *
     * @param messages 消息视图列表
     * @return true/false
     */
    @SuppressWarnings("unchecked")
    private boolean consume(List<MessageView> messages) {
        // 如果消费者组关闭则忽略该消息
        if (!ApplicationContextHolder.getProperty(this.group, boolean.class, true)) {
            if (log.isDebugEnabled()) {
                log.debug("RocketMQ message skipping: {}, {}", this.group, messages);
            }
            return true;
        }

        // 消息消费处理
        try {
            List<Pair<T, Session>> payloads = RocketMQContextHolder.deserialize(
                    messages, this.type, (payload, session) -> Pair.of((T) payload, session)
            );
            return this.listener.consume(payloads);
        } catch (Throwable t) {
            log.error("RocketMQ message consume failed: {}, {}", this.group, messages, t);
        }
        return false;
    }

    /**
     * 确认消息
     *
     * @param messages 消息视图列表
     */
    private void acknowledge(List<MessageView> messages) {
        for (MessageView message : messages) {
            try {
                this.consumer.ack(message);
            } catch (Throwable t) {
                if (t.getCause() instanceof InterruptedException) {
                    Thread.currentThread().interrupt();
                } else {
                    log.error("RocketMQ message acknowledge failed: {}, {}", this.group, message, t);
                }
            }
        }
    }

    /**
     * 启动消费线程
     *
     * @return 消费线程实例
     */
    private Thread startup() {
        return ThreadUtils.run(() -> {
            while (this.running && !Thread.currentThread().isInterrupted()) {
                List<MessageView> messages = this.receive();
                if (ObjectUtils.notEmpty(messages) && this.consume(messages)) {
                    this.acknowledge(messages);
                }
            }
        });
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull ClientServiceProvider provider,
                           @NonNull ClientConfiguration configuration, @NonNull RocketMQListener<T> listener,
                           @NonNull RocketMQConsumer annotation) {
        // 参数配置有效性校验
        AssertUtils.arg(StringUtils.notEmpty(annotation.topic()), () -> "Consumer topic must not be empty");
        AssertUtils.arg(StringUtils.notEmpty(annotation.group()), () -> "Consumer group must not be empty");
        AssertUtils.arg(annotation.batch() > 0, () -> "Consumer batch must be greater than 0");
        AssertUtils.arg(annotation.duration() > 0, () -> "Consumer duration must be greater than 0");
        AssertUtils.arg(annotation.interval() > 0, () -> "Consumer interval must be greater than 0");
        AssertUtils.arg(StringUtils.notEmpty(annotation.tag()), () -> "Consumer tag must not be empty");

        // 初始化消费者实例
        this.type = type;
        this.batch = annotation.batch();
        this.listener = listener;
        this.duration = Duration.ofSeconds(annotation.duration());
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.group = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.group()));
        String topic = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.topic()));
        Duration interval = Duration.ofSeconds(annotation.interval());
        FilterExpression filter = new FilterExpression(annotation.tag(), FilterExpressionType.TAG);
        SimpleConsumerBuilder builder = provider.newSimpleConsumerBuilder().setClientConfiguration(configuration)
                .setConsumerGroup(this.group).setAwaitDuration(interval)
                .setSubscriptionExpressions(Collections.singletonMap(topic, filter));
        try {
            this.consumer = builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }

        // 启动消费线程
        this.thread = this.startup();
    }

    @Override
    public void destroy() throws Exception {
        this.running = false;
        try {
            if (this.thread != null) {
                ThreadUtils.stop(this.thread);
            }
        } finally {
            if (this.consumer != null) {
                try {
                    this.consumer.close();
                } catch (Throwable t) {
                    log.warn("RocketMQ consumer shutdown failed: {}, {}", this.group, t.getMessage());
                }
            }
        }
    }
}
