package com.chelvc.framework.rocketmq.consumer;

import java.lang.reflect.Type;
import java.util.Collections;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.consumer.ConsumeResult;
import org.apache.rocketmq.client.apis.consumer.FilterExpression;
import org.apache.rocketmq.client.apis.consumer.FilterExpressionType;
import org.apache.rocketmq.client.apis.consumer.PushConsumer;
import org.apache.rocketmq.client.apis.consumer.PushConsumerBuilder;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.springframework.core.env.Environment;

/**
 * 消息单个消费监听器容器
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class SingleRocketMQListenerContainer<T> implements RocketMQListenerContainer<T> {
    private Type type;
    private String group;
    private PushConsumer consumer;
    private RocketMQListener<T> listener;

    /**
     * 消费消息
     *
     * @param message 消息视图
     * @return 消费结果
     */
    @SuppressWarnings("unchecked")
    private ConsumeResult consume(MessageView message) {
        // 如果消费者组关闭则忽略该消息
        if (!ApplicationContextHolder.getProperty(this.group, boolean.class, true)) {
            if (log.isDebugEnabled()) {
                log.debug("RocketMQ message skipping: {}, {}", this.group, message);
            }
            return ConsumeResult.SUCCESS;
        }

        // 消息消费处理
        try {
            return RocketMQContextHolder.deserialize(message, this.type, payload -> {
                this.listener.consume((T) payload);
                return ConsumeResult.SUCCESS;
            });
        } catch (Throwable t) {
            log.error("RocketMQ message consume failed: {}, {}", this.group, message, t);
            return ConsumeResult.FAILURE;
        }
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull ClientServiceProvider provider,
                           @NonNull ClientConfiguration configuration, @NonNull RocketMQListener<T> listener,
                           @NonNull RocketMQConsumer annotation) {
        // 参数配置有效性校验
        AssertUtils.arg(StringUtils.notEmpty(annotation.topic()), () -> "Consumer topic must not be empty");
        AssertUtils.arg(StringUtils.notEmpty(annotation.group()), () -> "Consumer group must not be empty");
        AssertUtils.arg(StringUtils.notEmpty(annotation.tag()), () -> "Consumer tag must not be empty");
        AssertUtils.arg(annotation.concurrency() > 0, () -> "Consumer concurrency must be greater than 0");

        // 初始化消费者实例
        this.type = type;
        this.listener = listener;
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.group = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.group()));
        String topic = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.topic()));
        FilterExpression filter = new FilterExpression(annotation.tag(), FilterExpressionType.TAG);
        PushConsumerBuilder builder = provider.newPushConsumerBuilder().setClientConfiguration(configuration)
                .setConsumerGroup(this.group).setConsumptionThreadCount(annotation.concurrency())
                .setSubscriptionExpressions(Collections.singletonMap(topic, filter)).setMessageListener(this::consume);
        try {
            this.consumer = builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void destroy() throws Exception {
        if (this.consumer != null) {
            try {
                this.consumer.close();
            } catch (Throwable t) {
                log.warn("RocketMQ consumer shutdown failed: {}, {}", this.group, t.getMessage());
            }
        }
    }
}
