package com.chelvc.framework.wechat.context;

import java.io.StringReader;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import java.util.function.Function;
import javax.crypto.Cipher;
import javax.xml.parsers.DocumentBuilderFactory;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.NumberUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.wechat.WechatAccessToken;
import com.chelvc.framework.wechat.WechatPublicHandler;
import com.chelvc.framework.wechat.WechatResponse;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.github.wxpay.sdk.WXPayUtil;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * 微信上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
public final class WechatContextHolder implements ApplicationListener<ApplicationStartedEvent> {
    /**
     * 工具类初始化标记
     */
    private static volatile boolean INITIALIZED;

    /**
     * 获取访问令牌接口地址
     */
    private static final String ACCESS_TOKEN_URL = "https://api.weixin.qq.com/cgi-bin/stable_token";

    /**
     * 应用标识/密钥映射表
     */
    private static final Map<String, String> APPID_SECRET_MAPPING = Maps.newHashMap();

    /**
     * 应用标识/令牌/过期时间戳映射表
     */
    private static final Map<String, Pair<String, Long>> APPID_TOKEN_MAPPING = Maps.newConcurrentMap();

    /**
     * 微信公众号处理器映射表
     */
    private static volatile Map<String, WechatPublicHandler> WECHAT_PUBLIC_HANDLER_MAPPING;

    private WechatContextHolder() {
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        initialize(ApplicationContextHolder.getBean(WechatProperties.class));
    }

    /**
     * 刷新访问令牌
     *
     * @param appid AppID
     * @return 访问令牌/过期时间戳
     */
    private static Pair<String, Long> refreshAccessToken(String appid) {
        String secret = APPID_SECRET_MAPPING.get(appid);
        AssertUtils.state(StringUtils.notEmpty(secret), () -> "Wechat secret not found: " + appid);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        Map<?, ?> body = ImmutableMap.of("grant_type", "client_credential", "appid", appid, "secret", secret);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        WechatAccessToken response = exchange(ACCESS_TOKEN_URL, HttpMethod.POST, entity, WechatAccessToken.class);
        if (log.isDebugEnabled()) {
            log.debug("Wechat access token: {}, {}", appid, response.getToken());
        }
        // 令牌有效时间 = 微信平台规定时间 - 5分钟
        long duration = (response.getDuration() - 300) * 1000;
        return Pair.of(response.getToken(), System.currentTimeMillis() + duration);
    }

    /**
     * 微信上下文工具初始化
     *
     * @param properties 微信配置属性
     */
    public synchronized static void initialize(@NonNull WechatProperties properties) {
        if (INITIALIZED) {
            return;
        }
        INITIALIZED = true;

        // 初始化应用标识/密钥映射表
        if (ObjectUtils.notEmpty(properties.getApplets())) {
            properties.getApplets().forEach(config -> APPID_SECRET_MAPPING.put(config.getAppid(), config.getSecret()));
        }
        if (ObjectUtils.notEmpty(properties.getPublics())) {
            properties.getPublics().forEach(config -> APPID_SECRET_MAPPING.put(config.getAppid(), config.getSecret()));
        }

        // 初始化令牌刷新服务线程
        if (ObjectUtils.notEmpty(APPID_SECRET_MAPPING)) {
            ThreadUtils.run(() -> {
                while (!Thread.currentThread().isInterrupted()) {
                    APPID_SECRET_MAPPING.forEach((appid, secret) -> {
                        Pair<String, Long> pair = APPID_TOKEN_MAPPING.get(appid);
                        if (pair == null || pair.getRight() <= System.currentTimeMillis()) {
                            try {
                                APPID_TOKEN_MAPPING.put(appid, refreshAccessToken(appid));
                            } catch (Exception e) {
                                log.warn("Wechat access token refresh failed: {}, {}", appid, e.getMessage());
                            }
                        }
                    });
                    ThreadUtils.sleep(60000);
                }
            });
        }
    }

    /**
     * 判断字符串是否是微信订单ID
     *
     * @param identity ID字符串
     * @return true/false
     */
    public static boolean isOrderIdentity(String identity) {
        return StringUtils.length(identity) == 28 && StringUtils.isDigit(identity);
    }

    /**
     * 获取微信访问令牌
     *
     * @param appid AppID
     * @return 访问令牌
     */
    public static String getAccessToken(@NonNull String appid) {
        Pair<String, Long> token = APPID_TOKEN_MAPPING.get(appid);
        if (token == null) {
            token = APPID_TOKEN_MAPPING.computeIfAbsent(appid, WechatContextHolder::refreshAccessToken);
        }
        return token.getLeft();
    }

    /**
     * 获取微信公众号处理器
     *
     * @return 处理器名称/实例映射表
     */
    public static Map<String, WechatPublicHandler> getWechatPublicHandlers() {
        if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
            synchronized (WechatPublicHandler.class) {
                if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
                    Map<String, WechatPublicHandler> handlers =
                            ApplicationContextHolder.getBeans(WechatPublicHandler.class);
                    if (ObjectUtils.isEmpty(handlers)) {
                        WECHAT_PUBLIC_HANDLER_MAPPING = Collections.emptyMap();
                    } else {
                        WECHAT_PUBLIC_HANDLER_MAPPING = Collections.unmodifiableMap(Maps.newHashMap(handlers));
                    }
                }
            }
        }
        return WECHAT_PUBLIC_HANDLER_MAPPING;
    }

    /**
     * 获取微信公众号处理器
     *
     * @param id 微信公众号配置ID
     * @return 微信公众号处理器
     */
    public static WechatPublicHandler getWechatPublicHandler(@NonNull String id) {
        WechatPublicHandler handler = getWechatPublicHandlers().get(id);
        AssertUtils.state(handler != null, () -> "Wechat public handler not found: " + id);
        return handler;
    }

    /**
     * 将键/值对转换成xml
     *
     * @param map 键/值对
     * @return xml
     */
    public static String map2xml(@NonNull Map<String, String> map) {
        try {
            return WXPayUtil.mapToXml(map);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将xml转换成键/值对
     *
     * @param xml XML
     * @return 键/值对
     */
    public static Map<String, String> xml2map(@NonNull String xml) {
        // 替换WXPayUtil.xmlToMap()方法，解决中文乱码问题
        InputSource source = new InputSource(new StringReader(xml));
        try {
            Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(source);
            document.getDocumentElement().normalize();
            NodeList nodes = document.getDocumentElement().getChildNodes();
            int length = nodes == null ? 0 : nodes.getLength();
            if (length > 0) {
                Map<String, String> map = Maps.newHashMapWithExpectedSize(length);
                for (int i = 0; i < length; i++) {
                    Node node = nodes.item(i);
                    if (node.getNodeType() == 1) {
                        Element element = (Element) node;
                        map.put(element.getNodeName(), element.getTextContent());
                    }
                }
                return map.isEmpty() ? Collections.emptyMap() : Collections.unmodifiableMap(map);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return Collections.emptyMap();
    }

    /**
     * 将金额数字转换成字符串
     *
     * @param amount 金额数字
     * @return 金额字符串形式
     */
    public static String amount2string(BigDecimal amount) {
        return NumberUtils.format(NumberUtils.multiply100(amount), true);
    }

    /**
     * 将金额字符串转换成数字
     *
     * @param amount 金额字符串
     * @return 金额数字
     */
    public static BigDecimal string2amount(String amount) {
        long value = StringUtils.ifEmpty(amount, Long::parseLong, 0L);
        return NumberUtils.divide100(BigDecimal.valueOf(value), 2);
    }

    /**
     * 参数签名
     *
     * @param key        签名密钥
     * @param parameters 签名参数
     * @return 签名信息
     */
    public static String sign(@NonNull String key, @NonNull Map<String, String> parameters) {
        try {
            return WXPayUtil.generateSignature(parameters, key);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 解码手机号
     *
     * @param ciphertext 会话信息密文
     * @param secret     密钥
     * @param iv         偏移量
     * @return 手机号明文
     */
    public static String mobile(String ciphertext, @NonNull String secret, @NonNull String iv) {
        if (StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }
        byte[] bs = CodecUtils.decodeBase64(secret), bi = CodecUtils.decodeBase64(iv);
        Cipher cipher = AESUtils.getDecrypter(AESUtils.CBC_PKCS5PADDING, bs, bi);
        Map<?, ?> map = JacksonUtils.deserialize(CodecUtils.decrypt(cipher, ciphertext), Map.class);
        return map == null ? null : (String) map.get("phoneNumber");
    }

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @return 明文
     */
    public static String decrypt(String ciphertext, @NonNull String secret) {
        if (StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }

        // 解密数据
        byte[] key = CodecUtils.decodeBase64(secret), iv = Arrays.copyOfRange(key, 0, 16);
        Cipher cipher = AESUtils.getDecrypter(AESUtils.CBC_NOPADDING, key, iv);
        byte[] original = CodecUtils.codec(cipher, CodecUtils.decodeBase64(ciphertext));

        // 删除解密后明文的补位字符
        int pad = original[original.length - 1];
        if (pad < 1 || pad > 32) {
            pad = 0;
        }
        byte[] bytes = Arrays.copyOfRange(original, 0, original.length - pad);

        // 分离16位随机字符串,网络字节序和AppId
        int length = 0;
        for (int i = 16; i < 20; i++) {
            length <<= 8;
            length |= bytes[i] & 0xff;
        }
        return new String(Arrays.copyOfRange(original, 20, 20 + length), StandardCharsets.UTF_8);
    }

    /**
     * 执行接口请求
     *
     * @param function 请求回调函数
     * @param <T>      结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T execute(@NonNull Function<RestTemplate, T> function) {
        T response = RestContextHolder.execute(function);
        int status = ObjectUtils.ifNull(response.getErrcode(), 0);
        if (status != 0) {
            throw new IllegalStateException(status + ", " + response.getErrmsg());
        }
        return response;
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T get(@NonNull String url, @NonNull Class<T> type,
                                                   @NonNull Object... parameters) {
        return execute(rest -> rest.getForObject(url, type, parameters));
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T get(@NonNull String url, @NonNull Class<T> type,
                                                   @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.getForObject(url, type, parameters));
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                                                    @NonNull Object... parameters) {
        return execute(rest -> rest.postForObject(url, request, type, parameters));
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                                                    @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.postForObject(url, request, type, parameters));
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity, @NonNull Class<T> type,
                                                        @NonNull Object... parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity, @NonNull Class<T> type,
                                                        @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity,
                                                        @NonNull ParameterizedTypeReference<T> type,
                                                        @NonNull Object... parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity,
                                                        @NonNull ParameterizedTypeReference<T> type,
                                                        @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }
}
