package com.chelvc.framework.wechat.support;

import java.util.List;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.PayChannel;
import com.chelvc.framework.wechat.WechatOrder;
import com.chelvc.framework.wechat.WechatPaymentCallback;
import com.chelvc.framework.wechat.WechatPaymentHandler;
import com.chelvc.framework.wechat.WechatPaymentProcessor;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 微信支付操作默认实现
 *
 * @author Woody
 * @date 2024/6/19
 */
@Slf4j
public class DefaultWechatPaymentHandler implements WechatPaymentHandler {
    private final List<WechatPaymentProcessor> processors;

    public DefaultWechatPaymentHandler(@NonNull List<WechatPaymentProcessor> processors) {
        AssertUtils.arg(!processors.isEmpty(), () -> "Wechat payment processors must not be empty");
        this.processors = Lists.newArrayList(processors);
    }

    @Override
    public boolean supports(@NonNull String name, PayChannel channel) {
        for (WechatPaymentProcessor processor : this.processors) {
            if (Objects.equals(name, processor.getName()) && (channel == null || channel == processor.getChannel())) {
                return true;
            }
        }
        return false;
    }

    @Override
    public WechatPaymentProcessor getProcessor(@NonNull String name, PayChannel channel) {
        WechatPaymentProcessor optional = null;
        String merchant = ApplicationContextHolder.getProperty("wechat.payment." + name + ".merchant");
        for (WechatPaymentProcessor processor : this.processors) {
            if (Objects.equals(name, processor.getName()) && (channel == null || channel == processor.getChannel())) {
                if (optional == null) {
                    optional = processor;
                }

                // 优先使用指定商户号支付配置
                if (StringUtils.isEmpty(merchant) || Objects.equals(merchant, processor.getMchid())) {
                    return processor;
                }
            }
        }
        if (optional == null) {
            throw new IllegalArgumentException("Wechat payment processor not found: " + name);
        }
        return optional;
    }

    @Override
    public WechatPaymentCallback callback(@NonNull String name, @NonNull String content) {
        // 首次处理（使用匹配的处理器）
        WechatPaymentProcessor primary = this.getProcessor(name, null);
        WechatPaymentCallback callback = primary.callback(content);
        if (Objects.nonNull(callback)) {
            return callback;
        }

        // 兼容处理，防止在支付配置切换过程中导致支付回调失败的情况
        for (WechatPaymentProcessor processor : this.processors) {
            if (processor != primary && Objects.equals(name, processor.getName())
                    && Objects.nonNull(callback = processor.callback(content))) {
                return callback;
            }
        }
        throw new RuntimeException("Wechat payment callback failed: " + content);
    }

    @Override
    public WechatOrder getOrder(@NonNull String name, @NonNull String identity) {
        // 首次处理（使用匹配的处理器）
        WechatPaymentProcessor primary = this.getProcessor(name, null);
        WechatOrder order = primary.getOrder(identity);
        if (Objects.nonNull(order)) {
            return order;
        }

        // 兼容处理，防止在支付配置切换过程中导致获取订单失败的情况
        for (WechatPaymentProcessor processor : this.processors) {
            if (processor != primary && Objects.equals(name, processor.getName())
                    && Objects.nonNull(order = processor.getOrder(identity))) {
                return order;
            }
        }
        return null;
    }

    @Override
    public boolean closeOrder(@NonNull String name, @NonNull String identity) {
        // 首次处理（使用匹配的处理器）
        WechatPaymentProcessor primary = this.getProcessor(name, null);
        if (primary.closeOrder(identity)) {
            return true;
        }

        // 兼容处理，防止在支付配置切换过程中导致订单关闭失败的情况
        for (WechatPaymentProcessor processor : this.processors) {
            if (processor != primary && Objects.equals(name, processor.getName()) && processor.closeOrder(identity)) {
                return true;
            }
        }
        return false;
    }
}
