package com.chelvc.framework.wechat.support;

import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.util.HostUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.PayChannel;
import com.chelvc.framework.wechat.PayMode;
import com.chelvc.framework.wechat.WechatOrder;
import com.chelvc.framework.wechat.WechatOrderResponse;
import com.chelvc.framework.wechat.WechatPayRequest;
import com.chelvc.framework.wechat.WechatPaymentCallback;
import com.chelvc.framework.wechat.WechatPaymentProcessor;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.github.wxpay.sdk.WXPayUtil;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.MessageDigestAlgorithms;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;

/**
 * 微信支付处理器标准实现
 *
 * @author Woody
 * @date 2024/9/10
 */
@Slf4j
public class StandardWechatPaymentProcessor implements WechatPaymentProcessor {
    /**
     * 订单查询接口地址
     */
    private static final String ORDER_QUERY_URL = "https://api.mch.weixin.qq.com/pay/orderquery";

    /**
     * 订单关闭接口地址
     */
    private static final String ORDER_CLOSE_URL = "https://api.mch.weixin.qq.com/pay/closeorder";

    /**
     * 微信统一下单接口地址
     */
    private static final String UNIFIED_ORDER_URL = "https://api.mch.weixin.qq.com/pay/unifiedorder";

    /**
     * 支付回调响应内容
     */
    private static final String CALLBACK_RESPONSE =
            "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>";

    @Getter
    private final String name;
    @Getter
    private final String appid;
    @Getter
    private final String mchid;
    @Getter
    private final PayMode mode;
    private final String mchkey;
    @Getter
    private final String callback;
    @Getter
    private final PayChannel channel;

    public StandardWechatPaymentProcessor(@NonNull WechatProperties.Payment properties) {
        this.name = properties.getName();
        this.mode = properties.getMode();
        this.appid = properties.getAppid();
        this.mchid = properties.getMchid();
        this.mchkey = properties.getMchkey();
        this.channel = properties.getChannel();
        this.callback = properties.getCallback();
    }

    /**
     * 参数签名
     *
     * @param parameters 签名参数
     * @return 签名信息
     */
    private String sign(@NonNull Map<String, String> parameters) {
        return WechatContextHolder.sign(this.mchkey, parameters);
    }

    /**
     * 执行接口调用
     *
     * @param url        接口地址
     * @param parameters 接口参数
     * @return 响应结果
     */
    private Map<String, String> execute(String url, Map<String, String> parameters) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.TEXT_XML);
        String body = WechatContextHolder.map2xml(parameters);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        ResponseEntity<String> response =
                RestContextHolder.execute(rest -> rest.exchange(url, HttpMethod.POST, entity, String.class));
        return WechatContextHolder.xml2map(Objects.requireNonNull(response.getBody()));
    }

    @Override
    public WechatPaymentCallback callback(@NonNull String content) {
        Map<String, String> payment;
        try {
            payment = WechatContextHolder.xml2map(content);
        } catch (Exception e) {
            log.error("Wechat payment callback analysing failed: {}", content, e);
            return null;
        }
        if (log.isDebugEnabled()) {
            log.debug("Wechat payment callback: {}", payment);
        }
        if (Objects.isNull(payment) || !Objects.equals(payment.get("mch_id"), this.mchid)) {
            return null;
        } else if (!Objects.equals(payment.get("return_code"), "SUCCESS")
                || !Objects.equals(payment.get("result_code"), "SUCCESS")
                || !Objects.equals(payment.get("sign"), this.sign(payment))) {
            throw new RuntimeException("Wechat payment callback verify failed: " + payment);
        }
        return WechatOrder.of(WechatPaymentCallback.builder().response(CALLBACK_RESPONSE).build(), payment);
    }

    @Override
    public WechatOrderResponse unifiedorder(@NonNull WechatPayRequest request) {
        // 调用微信接口
        Map<String, String> parameters = Maps.newHashMap();
        parameters.put("appid", this.appid);
        parameters.put("mch_id", this.mchid);
        parameters.put("body", request.getComment());
        parameters.put("out_trade_no", request.getOrder());
        if (StringUtils.notEmpty(request.getContext())) {
            parameters.put("attach", request.getContext());
        }
        parameters.put("fee_type", "CNY");
        parameters.put("total_fee", WechatContextHolder.amount2string(request.getAmount()));
        parameters.put("spbill_create_ip", HostUtils.LOCAL_ADDRESS);
        parameters.put("notify_url", this.callback);
        parameters.put("trade_type", StringUtils.toString(this.mode));
        parameters.put("nonce_str", WXPayUtil.generateNonceStr());
        if (this.mode == PayMode.MWEB) {
            // 如果是H5支付则需要指定scene_info，场景信息
            parameters.put("scene_info", request.getScene());
        } else if (this.mode == PayMode.JSAPI) {
            // 如果是小程序支付则需要指定openid
            parameters.put("openid", request.getOpenid());
        }
        parameters.put("sign", this.sign(parameters));
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Wechat unifiedorder request: {}", parameters);
        }
        Map<String, String> result = this.execute(UNIFIED_ORDER_URL, parameters);
        if (debug) {
            log.debug("Wechat unifiedorder response: {}", result);
        }
        if (!Objects.equals(result.get("return_code"), "SUCCESS")) {
            throw new RuntimeException("Wechat unifiedorder failed: " + result.get("return_msg"));
        } else if (!Objects.equals(result.get("result_code"), "SUCCESS")) {
            throw new RuntimeException("Wechat unifiedorder failed: " + result.get("err_code_des"));
        } else if (!Objects.equals(result.get("sign"), this.sign(result))) {
            String message = ApplicationContextHolder.getMessage("Signature.Invalid");
            throw new RuntimeException("Wechat unifiedorder failed: " + message);
        }

        // 构建响应结果
        String nonce = WXPayUtil.generateNonceStr();
        String timestamp = String.valueOf(System.currentTimeMillis() / 1000).substring(0, 10);
        WechatOrderResponse order = WechatOrderResponse.builder().mode(this.mode).appid(this.appid)
                .mchid(this.mchid).nonce(nonce).prepayid(result.get("prepay_id")).qrcode(result.get("code_url"))
                .redirect(result.get("mweb_url")).channel(this.channel).timestamp(timestamp).build();
        if (this.mode == PayMode.MWEB || this.mode == PayMode.JSAPI) {
            order.setAlgorithm(MessageDigestAlgorithms.MD5);
            order.setPack("prepay_id=" + order.getPrepayid());

            // 设置数字签名
            Map<String, String> signature = Maps.newHashMapWithExpectedSize(5);
            signature.put("appId", order.getAppid());
            signature.put("nonceStr", order.getNonce());
            signature.put("package", order.getPack());
            signature.put("signType", order.getAlgorithm());
            signature.put("timeStamp", order.getTimestamp());
            order.setSign(this.sign(signature));
        } else {
            order.setPack("Sign=WXPay");

            // 设置数字签名
            Map<String, String> signature = Maps.newHashMapWithExpectedSize(6);
            signature.put("appid", order.getAppid());
            signature.put("partnerid", order.getMchid());
            signature.put("prepayid", order.getPrepayid());
            signature.put("noncestr", order.getNonce());
            signature.put("package", order.getPack());
            signature.put("timestamp", order.getTimestamp());
            order.setSign(this.sign(signature));
        }
        return order;
    }

    @Override
    public WechatOrder getOrder(@NonNull String identity) {
        Map<String, String> parameters = Maps.newHashMapWithExpectedSize(5);
        parameters.put("appid", this.appid);
        parameters.put("mch_id", this.mchid);
        parameters.put("nonce_str", WXPayUtil.generateNonceStr());
        if (WechatContextHolder.isOrderIdentity(identity)) {
            parameters.put("transaction_id", identity);
        } else {
            parameters.put("out_trade_no", identity);
        }
        parameters.put("sign", this.sign(parameters));
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Wechat order query request: {}", parameters);
        }
        Map<String, String> result = this.execute(ORDER_QUERY_URL, parameters);
        if (debug) {
            log.debug("Wechat order query response: {}", result);
        }
        if (!Objects.equals(result.get("return_code"), "SUCCESS")) {
            throw new RuntimeException("Wechat order query failed: " + result.get("return_msg"));
        } else if (!Objects.equals(result.get("result_code"), "SUCCESS")) {
            if (Objects.equals(result.get("err_code"), "ORDERNOTEXIST")) {
                return null;
            }
            throw new RuntimeException("Wechat order query failed: " + result.get("err_code_des"));
        } else if (!Objects.equals(result.get("sign"), this.sign(result))) {
            String message = ApplicationContextHolder.getMessage("Signature.Invalid");
            throw new RuntimeException("Wechat order query failed: " + message);
        }
        return WechatOrder.of(result);
    }

    @Override
    public boolean closeOrder(@NonNull String identity) {
        // 因微信平台对不存在的订单号进行关单仍能够关单成功
        // 故先获取订单信息并根据交易状态判断是否需要继续执行关单操作
        WechatOrder order = this.getOrder(identity);
        if (order == null) {
            return false;
        } else if (Objects.equals(order.getStatus(), "CLOSED")) {
            return true;
        }

        // 调用微信关单接口
        Map<String, String> parameters = Maps.newHashMapWithExpectedSize(5);
        parameters.put("appid", this.appid);
        parameters.put("mch_id", this.mchid);
        parameters.put("nonce_str", WXPayUtil.generateNonceStr());
        parameters.put("out_trade_no", identity);
        parameters.put("sign", this.sign(parameters));
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Wechat order close request: {}", parameters);
        }
        Map<String, String> result = this.execute(ORDER_CLOSE_URL, parameters);
        if (debug) {
            log.debug("Wechat order close response: {}", result);
        }
        if (!Objects.equals(result.get("return_code"), "SUCCESS")) {
            throw new RuntimeException("Wechat order close failed: " + result.get("return_msg"));
        } else if (!Objects.equals(result.get("result_code"), "SUCCESS")) {
            if (Objects.equals(result.get("err_code"), "ORDERCLOSED")) {
                return true;
            }
            throw new RuntimeException("Wechat order close failed: " + result.get("err_code_des"));
        } else if (!Objects.equals(result.get("sign"), this.sign(result))) {
            String message = ApplicationContextHolder.getMessage("Signature.Invalid");
            throw new RuntimeException("Wechat order close failed: " + message);
        }
        return true;
    }
}
